
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripOverlay()
{
    this.enabledObserver = null;
    this.flickerObserver = null;
    this.modeObserver = null;
    this.deepObserver = null;
    this.alwaysUseIndexObserver = null;
    this.loggingEnabledObserver = null;
    this.elementsInContextObserver = null;
    this.profileObserver = null;
//    this.saveObserver = null;

    this.appcontent = null;
    this.contextMenu = null;
    this.yaripMenupopup = null;
    this.panelContainer = null;
    this.stringbundle = null;

    this.doc = null;
    this.loader = null;
    this.handler = null;

    this.active = false;

    this.start = function(event)
    {
        if (event.button != 0) return;

        var doc = content.document;

        if (this.active)
        {
            this.stop(doc, false);
            return;
        }

        if (!yarip.enabled)
        {
            this.toggleEnabled(true);
            return;
        }

        if (!doc) return;

        if (!this.handler.start(doc))
        {
            this.stop(doc, false);
            return;
        }

        this.active = true;
        this.doc = doc;
        this.setStatus(doc, "started");
    }

    this.stop = function(doc, noStateChange)
    {
        if (!this.active || !this.handler) return false;

        if (this.handler.stop())
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
            yarip.setValue(yarip.RELOAD, new Date().getTime(), yarip.STRING); // notify
        }

        this.active = false;

        doc = doc ? doc : this.doc;
        if (!doc) return true;

        if (!noStateChange) this.setStatus(doc, "stopped");
        this.doc = null;

        return true;
    }

    this.whitelistXpath = function(doc, node)
    {
        this.toggleEnabled(true);

        if (!doc || !node) return;

        var address = yarip.getAddress(doc);
        if (!address) return;

        var xValue = yarip.calculateXPath(node);
        if (!xValue) return;

        var obj = {
            element: new YaripElement(xValue + "/descendant-or-self::*")
        }

        window.openDialog("chrome://yarip/content/whitelistdialog.xul", "whitelistdialog", "chrome,modal,resizable", obj);

        if (!obj.element) return;

        if (yarip.whitelistXpath(doc, address, obj.element.getXPath(), true, obj.element, true))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
            yarip.setValue(yarip.RELOAD, new Date().getTime(), yarip.STRING); // notify
        }
    }

    this.blacklistXpath = function(doc, node)
    {
        this.toggleEnabled(true);

        if (!doc || !node) return;

        var address = yarip.getAddress(doc);
        if (!address) return;

        var xValue = yarip.calculateXPath(node);
        if (!xValue) return;

        var obj = {
            element: new YaripElement(xValue)
        }

        window.openDialog("chrome://yarip/content/blacklistdialog.xul", "blacklistdialog", "chrome,modal,resizable", obj);

        if (!obj.element) return;

        if (yarip.blacklistXpath(doc, address, obj.element.getXPath(), true, obj.element.getForce(), obj.element, true))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
            yarip.setValue(yarip.RELOAD, new Date().getTime(), yarip.STRING); // notify
        }
    }

    this.removeTemporarily = function(doc, node)
    {
        this.toggleEnabled(true);

        if (!doc || !node) return;

        var address = yarip.getAddress(doc);
        if (!address) return;

        var xValue = yarip.calculateXPath(node);
        if (!xValue) return;

        yarip.blacklistXpath(doc, address, xValue, false, true);
    }

    this.style = function(doc, node)
    {
        this.toggleEnabled(true);

        if (!doc || !node) return;

        var address = yarip.getAddress(doc);
        if (!address) return;

        var xValue = yarip.calculateXPath(node);
        if (!xValue) return;

        var obj = {
            xValue: xValue,
            style: node.getAttribute("style")
        }
        if (obj.style) obj.style = obj.style.replace(yarip.endOutlineRegExp, "");

        window.openDialog("chrome://yarip/content/styledialog.xul", "styledialog", "chrome,modal,resizable", obj);

        if (!obj.xValue || obj.xValue == "") return;
        if (!obj.style || obj.style == "") return;

        if (yarip.style(doc, address, obj.xValue, obj.style, true, true))
        {
            yarip.map.sort(false);
            yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
            yarip.setValue(yarip.RELOAD, new Date().getTime(), yarip.STRING); // notify
        }
    }

    this.copyXPath = function(node)
    {
        if (!node) return;

        var xValue = yarip.calculateXPath(node);
        if (!xValue) return;

        var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString(xValue);
    }

    this.startHighlighting = function(node)
    {
        if (!node) return;

        this.handler.doc = node.ownerDocument;
        this.handler.element = node;
        this.handler.select();
    }

    this.stopHighlighting = function()
    {
        this.handler.doc = null;
        this.handler.reset();
    }

    this.createPage = function()
    {
        this.stop();
        yarip.undoObj = {}
        yarip.toggleStyle(false);
        var address = yarip.getAddress(content.document);
        if (address && !yarip.map.obj[address]) yarip.map.add(new YaripPage(yarip.getId(), address));
        yarip.map.sort(true);
        window.openDialog("chrome://yarip/content/pagedialog.xul", "pagedialog", "chrome,resizable", address);
    }

    this.managePages = function()
    {
        this.stop();
        yarip.map.sort(true);
        yarip.undoObj = {}
        yarip.toggleStyle(false);
        var aAddress = yarip.getAddressArray(content.document);
        var pageAddress = null;
        for (var a in aAddress)
        {
            pageAddress = a;
            break;
        }
        window.openDialog("chrome://yarip/content/pagedialog.xul", "pagedialog", "chrome,resizable", pageAddress);
    }

    this.toggleEnabled = function(enabled, force)
    {
        this.enabledObserver.disable();
        if (!yarip.toggleEnabled(enabled) && !force)
        {
            this.enabledObserver.enable();
            return;
        }
        this.enabledObserver.enable();

        document.getElementById("yarip-enabled-menuitem").setAttribute("checked", "" + yarip.enabled);
        this.setStatus(content.document, yarip.enabled ? "enabled" : "disabled");
    }

    this.updateEnabled = function()
    {
        this.toggleEnabled(yarip.getValue(yarip.ENABLED, true, yarip.BOOLEAN), true);
    }

    this.toggleNoFlicker = function(noFlicker, force)
    {
        this.flickerObserver.disable();
        if (!yarip.toggleNoFlicker(noFlicker) && !force)
        {
            this.flickerObserver.enable();
            return;
        }
        this.flickerObserver.enable();

        document.getElementById("yarip-flicker-menuitem").setAttribute("checked", "" + yarip.noFlicker);
    }

    this.updateNoFlicker = function()
    {
        this.toggleNoFlicker(yarip.getValue(yarip.FLICKER, false, yarip.BOOLEAN), true);
    }

    this.setMode = function(mode, clear, force, update)
    {
        var address = yarip.getAddress(content.document);

        this.modeObserver.disable();

        if (!update)
        {
            if (!yarip.setMode(mode) && !force)
            {
                this.modeObserver.enable();
                return;
            }
            this.modeObserver.enable();
        }
        else this.modeObserver.enable();

        if (clear)
        {
            document.getElementById("yarip-page-menuitem").setAttribute("checked", "false");
            document.getElementById("yarip-fqdn-menuitem").setAttribute("checked", "false");
            document.getElementById("yarip-sld-menuitem").setAttribute("checked", "false");
            document.getElementById("yarip-user-menuitem").setAttribute("checked", "false");
        }

        this.stop();
        this.toggleDeep(true/*, true*/);

//        var address = yarip.getAddress(content.document);
        switch (mode)
        {
            case yarip.PAGE_MODE:
                document.getElementById("yarip-page-menuitem").setAttribute("checked", "true");
                break;

            case yarip.FQDN_MODE:
                document.getElementById("yarip-fqdn-menuitem").setAttribute("checked", "true");
                break;

            case yarip.SLD_MODE:
                document.getElementById("yarip-sld-menuitem").setAttribute("checked", "true");
                break;

            case yarip.USER_MODE:
                document.getElementById("yarip-user-menuitem").setAttribute("checked", "true");
                if (update) return;

                if (!address) address = yarip.address ? yarip.address : "";
                address = prompt("Edit address", address);
                if (!address || address == "")
                {
                    this.setMode(yarip.FQDN_MODE, true);
                    return;
                }

                yarip.address = address;
                break;

            default:
                this.setMode(yarip.FQDN_MODE, true);
        }
    }

    this.updateMode = function()
    {
        this.setMode(yarip.getValue(yarip.MODE, yarip.FQDN_MODE, yarip.INTEGER), true, true, true);
    }

    this.toggleDeep = function(deep, force)
    {
        this.deepObserver.disable();
        if (!yarip.toggleDeep(deep) && !force)
        {
            this.deepObserver.enable();
            return;
        }
        else this.deepObserver.enable();

        document.getElementById("yarip-deep-menuitem").setAttribute("checked", "" + yarip.deep);
    }

    this.updateDeep = function()
    {
        this.toggleDeep(yarip.getValue(yarip.DEEP, true, yarip.BOOLEAN), true);
    }

    this.handleEvent = function(event)
    {
        if (!window || !("yarip" in window)) return;

        switch(event.type)
        {
            case "load":
                this.loader = new YaripLoader();
                this.handler = new YaripHandler(null, function() { yaripOverlay.stop(); });

                this.enabledObserver = new YaripObserver(
                    yarip.ENABLED_ROOT,
                    yarip.VALUE,
                    function() { yaripOverlay.updateEnabled(); }
                );
                this.flickerObserver = new YaripObserver(
                    yarip.FLICKER_ROOT,
                    yarip.VALUE,
                    function() { yaripOverlay.updateNoFlicker(); }
                );
                this.modeObserver = new YaripObserver(
                    yarip.MODE_ROOT,
                    yarip.VALUE,
                    function() { yaripOverlay.updateMode(); }
                );
                this.deepObserver = new YaripObserver(
                    yarip.DEEP_ROOT,
                    yarip.VALUE,
                    function() { yaripOverlay.updateDeep(); }
                );
                this.alwaysUseIndexObserver = new YaripObserver(
                    yarip.INDEX_ROOT,
                    yarip.VALUE,
                    function() { yarip.setAlwaysUseIndex(yarip.getValue(yarip.INDEX, false, yarip.BOOLEAN)); }
                );
                this.loggingEnabledObserver = new YaripObserver(
                    yarip.LOGGING_ROOT,
                    yarip.VALUE,
                    function() { yarip.setLoggingEnabled(yarip.getValue(yarip.LOGGING, false, yarip.BOOLEAN)); }
                );
                this.elementsInContextObserver = new YaripObserver(
                    yarip.ELEMENTS_ROOT,
                    yarip.VALUE,
                    function() { yarip.setElementsInContext(yarip.getValue(yarip.ELEMENTS, 4, yarip.INTEGER)); }
                );
                this.profileObserver = new YaripObserver(
                    yarip.PROFILE_ROOT,
                    yarip.VALUE,
                    function() { yarip.setProfile(yarip.getValue(yarip.PROFILE, "", yarip.STRING)); }
                );
//                this.saveObserver = new YaripObserver(
//                    yarip.SAVE_ROOT,
//                    yarip.VALUE,
//                    function() { yarip.save(); }
//                );

                this.appcontent = document.getElementById("appcontent");
                this.contextMenu = document.getElementById("contentAreaContextMenu");
                this.yaripMenupopup = document.getElementById("yarip-menupopup");
                this.panelContainer = gBrowser.mPanelContainer;
                this.stringbundle = document.getElementById("yarip-overlay-stringbundle");

                this.updateEnabled();
                this.updateNoFlicker();
                this.updateMode();
//                this.setMode(yarip.FQDN_MODE, true, true);
                this.updateDeep();

                yarip.injectCSS("chrome://yarip/content/blacklist.css", true);

                if (this.appcontent) this.appcontent.addEventListener("DOMContentLoaded", this, false);
                if (this.contextMenu) this.contextMenu.addEventListener("popupshowing", this, false);
                if (this.panelContainer) this.panelContainer.addEventListener("select", this, false);
                window.removeEventListener("load", this, false);

                break;

            case "unload":
                yarip.save();

                this.enabledObserver.unregister();
                this.flickerObserver.unregister();
                this.modeObserver.unregister();
                this.deepObserver.unregister();
                this.alwaysUseIndexObserver.unregister();
                this.loggingEnabledObserver.unregister();
                this.elementsInContextObserver.unregister();
                this.profileObserver.unregister();
//                this.saveObserver.unregister();

                if (this.appcontent) this.appcontent.removeEventListener("DOMContentLoaded", this, false);
                if (this.contextMenu) this.contextMenu.removeEventListener("popupshowing", this, false);
                if (this.panelContainer) this.panelContainer.removeEventListener("select", this, false);
                window.removeEventListener("load", this, false);
                window.removeEventListener("unload", this, false);
//                window.removeEventListener("DOMMenuItemActive", this, false);
//                window.removeEventListener("keypress", this, false);

                break;

            case "DOMContentLoaded":
                this.stop();
                this.toggleDeep(true);

                var doc = event.originalTarget;
                if (!doc || !doc.body) return;

                if (!/^(text\/html|application\/xhtml\+xml)$/.test(doc.contentType))
                {
                    doc.body.setAttribute("status", "show");
                    return;
                }

                yarip.undoObj = {}

                // LOADER
                if (yarip.enabled && doc.nodeName == "#document")
                {
                    if (this.loader.load(doc)) this.setStatus(doc, "found");
                    else this.setStatus(doc);
                    if (yarip.mode == yarip.USER_MODE) this.setMode(yarip.FQDN_MODE, true);
                }

                doc.body.setAttribute("status", "show");

                break;

            case "popupshowing":
                this.stop();

                if (event.target.getAttribute("id") != "contentAreaContextMenu") return;
//                if (event.target.localName == "menupopup") return;

                // YARIP-MENU
                var menuitem = this.yaripMenupopup.firstChild;
                while (menuitem && menuitem.getAttribute("id") != "yarip-frame-menu")
                {
                    var next = menuitem.nextSibling;
                    this.yaripMenupopup.removeChild(menuitem);
                    menuitem = next;
                }
                var node = document.popupNode;
                var doc = node.ownerDocument;
                var hidden = !yarip.getAddress(doc);
                var hasMenus = false;
                if (!hidden)
                {
                    var frameMenu = document.getElementById("yarip-frame-menu");
                    var undoMenu = document.getElementById("yarip-undo-menupopup");

                    // ELEMENT-MENUES
                    var tmpNode = node;
                    for (var i = 0; i < yarip.elementsInContext && tmpNode && tmpNode.localName && !/^(html|body)$/i.test(tmpNode.localName); i++)
                    {
                        var menu = this.createMenu(tmpNode);
                        if (menu)
                        {
                            this.yaripMenupopup.insertBefore(menu, frameMenu);
                            hasMenus = true;
                        }
                        tmpNode = tmpNode.parentNode;
                    }

                    // FRAME-MENU
                    var inFrame = gContextMenu.inFrame;
                    if (inFrame)
                    {
                        var frame = node.ownerDocument.defaultView.frameElement;
                        frameMenu.setAttribute("label", frame.localName.toUpperCase());
                        if (!yarip.getAddress(frame.ownerDocument)) frameMenu.setAttribute("disabled", true);
                        else frameMenu.removeAttribute("disabled");
                    }
                    frameMenu.hidden = !inFrame;
                    hasMenus = hasMenus || inFrame;

                    // UNDO-MENU
                    var menuitem = undoMenu.firstChild;
                    while (menuitem && menuitem.localName != "menuseparator")
                    {
                        var next = menuitem.nextSibling;
                        undoMenu.removeChild(menuitem);
                        menuitem = next;
                    }
                    var found = false;
                    for (var k in yarip.undoObj)
                    {
                        var obj = yarip.undoObj[k];
                        if (!obj || obj.document != doc) continue;

                        var menuitem = document.createElement("menuitem");
                        menuitem.setAttribute("label", this.stringbundle.getFormattedString("undo-" + obj.type, [obj.xValue]));
                        menuitem.obj = obj;
                        menuitem.setAttribute("oncommand", "yaripOverlay.undo(this.obj, true)");
                        undoMenu.insertBefore(menuitem, undoMenu.firstChild);
                        found = true;
                    }
                    document.getElementById("yarip-undo-menu-sep").hidden = !hasMenus || !found;
                    document.getElementById("yarip-undo-menu").hidden = !found;
                    hasMenus = hasMenus || found;
                }

                document.getElementById("yarip-menu").hidden = !hasMenus;
                document.getElementById("yarip-menu-sep").hidden = !hasMenus;

                break;

            case "select":
                if (yarip.mode == yarip.USER_MODE) this.setMode(yarip.FQDN_MODE, true);
                this.setStatus(gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument);

                break;
        }
    }

    this.createMenu = function(node)
    {
        var menu = document.createElement("menu");
        menu.node = node;
        menu.setAttribute("label", node.localName.toUpperCase());
        menu.setAttribute("onmouseover", "yaripOverlay.startHighlighting(this.node)");
        menu.setAttribute("onmouseout", "yaripOverlay.stopHighlighting()");
        var menupopup = document.createElement("menupopup");
        menu.appendChild(menupopup);

        var wMenuitem = document.createElement("menuitem");
        wMenuitem.node = node;
        wMenuitem.setAttribute("label", this.stringbundle.getString("menuitemElementWhitelist"));
        wMenuitem.setAttribute("oncommand", "yaripOverlay.whitelistXpath(document.popupNode.ownerDocument, this.node)");
        menupopup.appendChild(wMenuitem);

        var bMenuitem = document.createElement("menuitem");
        bMenuitem.node = node;
        bMenuitem.setAttribute("label", this.stringbundle.getString("menuitemElementBlacklist"));
        bMenuitem.setAttribute("oncommand", "yaripOverlay.blacklistXpath(document.popupNode.ownerDocument, this.node)");
        menupopup.appendChild(bMenuitem);

        var sMenuitem = document.createElement("menuitem");
        sMenuitem.node = node;
        sMenuitem.setAttribute("label", this.stringbundle.getString("menuitemStyle"));
        sMenuitem.setAttribute("oncommand", "yaripOverlay.style(document.popupNode.ownerDocument, this.node)");
        menupopup.appendChild(sMenuitem);

        var rMenuitem = document.createElement("menuitem");
        rMenuitem.node = node;
        rMenuitem.setAttribute("label", this.stringbundle.getString("menuitemRemoveTemporarily"));
        rMenuitem.setAttribute("oncommand", "yaripOverlay.removeTemporarily(document.popupNode.ownerDocument, this.node)");
        menupopup.appendChild(rMenuitem);

        var cMenuitem = document.createElement("menuitem");
        cMenuitem.node = node;
        cMenuitem.setAttribute("label", this.stringbundle.getString("menuitemCopyxpath"));
        cMenuitem.setAttribute("oncommand", "yaripOverlay.copyXPath(this.node)");
        menupopup.appendChild(cMenuitem);

        var found = false;
        if (node.attributes)
        {
            var aMenu = document.createElement("menu");
            aMenu.setAttribute("label", this.stringbundle.getString("menuAttribute"));

            var aMenupopup = document.createElement("menupopup");
            aMenu.appendChild(aMenupopup);

            for (var i = 0; i < node.attributes.length; i++)
            {
                var attribute = node.attributes[i];
                if (!attribute || attribute.value == "") continue;

                switch (attribute.name)
                {
                    case "status":
                        if (/^(whitelisted|blacklisted|styled)$/.test(attribute.value)) continue;

                    case "style":
                        if (yarip.outlineRegExp.test(attribute.value)) continue;
                }

                var menuitem = document.createElement("menuitem");
                menuitem.setAttribute("label", attribute.name.toLowerCase());
                menuitem.attribute = attribute;
                menuitem.setAttribute("oncommand", "yaripOverlay.blacklistXpath(document.popupNode.ownerDocument, this.attribute)");
                aMenupopup.insertBefore(menuitem, aMenupopup.firstChild);
                found = true;
            }

            if (found) menupopup.insertBefore(aMenu, sMenuitem);
        }

        return menu;
    }

    this.undoAll = function(doc)
    {
        if (!doc) return;

        for (var k in yarip.undoObj)
        {
            var obj = yarip.undoObj[k];
            if (obj && obj.document == doc) this.undo(obj);
        }

        yarip.setValue(yarip.RELOAD, new Date().getTime(), yarip.STRING); // notify
    }

    this.undo = function(obj, reload)
    {
        if (!obj) return;

        yarip.resetOnAddress(obj);
        if (reload) yarip.setValue(yarip.RELOAD, new Date().getTime(), yarip.STRING); // notify
    }

    this.setStatus = function(doc, status)
    {
        if (!doc) return;

        if (yarip.enabled)
        {
            if (!status)
            {
                status = doc.defaultView.yaripStatus;
                switch (status)
                {
                    case "enabled":
                    case "stopped":
                    case "found":
                        break;

                    case "disabled":
                    case "started":
                        if (this.active) break;

                    default:
                        status = "enabled";
                }
            }
        }
        else status = "disabled";

        if (doc.defaultView) doc.defaultView.yaripStatus = status;
        if (doc == gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument)
        {
            document.getElementById("yarip-statusbarpanel").setAttribute("status", status);
            document.getElementById("yarip-statusbarpanel").setAttribute("tooltiptext", this.stringbundle.getString("status-" + status));
        }
    }
}
